/*
GdiPlusRadar.h
(c) 2004 Arne Bartels arne.bartels@nwn.de
*/

#ifndef	GDIPLUSRADAR_H
#define	GDIPLUSRADAR_H



#ifndef	ULONG_PTR 
	#define		ULONG_PTR unsigned int
#endif

#ifndef UNICODE
#define		UNICODE
#endif

//Check also:
//http://msdn.microsoft.com/library/default.asp?url=/library/en-us/gdicpp/GDIPlus/GDIPlusreference.asp

#include	<gdiplus.h>
using		namespace	Gdiplus;



#define		INITIAL_RANGE		(10*1.5)
#define		MAX_RANGE			(40*1.5)
#define		MIN_RANGE			(.001)
#define		MAX_NUM_AIRCRAFT	100
#define		MAX_RANGE_AIRCRAFT	(MAX_RANGE*1.2)
#define		PEN_WIDTH			0.5f
#define		TAG_SIZE			1.5f
#define		PAD_SIZE			2.125f
#define		DOT_SIZE			0.5f
#define		TRACK_ANGLE_LEN		4.0
#define		FONT_SIZE			2.25f
#define		ABVSTR_OFFSET_X		0.0f
#define		ABVSTR_OFFSET_Y		-5.5f
#define		BLWSTR_OFFSET_X		0.0f
#define		BLWSTR_OFFSET_Y		2.0f
#define		POINTER_OFFSET		3.0f

#define		FONT_TYPE			L"Arial"

#define		HALF_WIDTH			50.0f
#define		HALF_HEIGHT			50.0f

#define		DETAIL_DOT				BIT0	//1
#define		DETAIL_SYMBOL_OUTLINE	BIT1	//2
#define		DETAIL_SYMBOL_FILLED	BIT2	//4
#define		DETAIL_ALTITUDE			BIT3	//8
#define		DETAIL_ID				BIT4	//16




class GdiPlusRadar  
{
protected:
	Graphics*				m_graphics;
	float					m_symbol_factor;
	WCHAR					m_symbolfont[255+1];
private:


	ULONG_PTR				m_gdiplustoken;


	double					m_crs_x;
	double					m_crs_y;
	double					m_crs_dist;

	TrafficData*			m_traffic_data;
	tAircraftData			m_tooltip_data;

	ARGB					m_other_argb;
	ARGB					m_proxy_argb;
	ARGB					m_ta_argb;
	ARGB					m_ra_argb;

	int						m_other_detail;
	int						m_proxy_detail;
	int						m_ta_detail;
	int						m_ra_detail;
	int						m_pad_symbol;

	double					m_resolution;
	float					m_size_x;
	float					m_size_y;
	float					m_scale_x;
	float					m_scale_y;
	float					m_centre_x;
	float					m_centre_y;

	int						m_max_dsp_act;
	int						m_dsp_act_cnt;
	int						m_max_ind_dsp;
	

private:
	virtual	void	DrawSymbol		(float x, float y,int detail,tAircraftData*	pai_data,FLOAT64	true_hdg,Color& color,WCHAR	*altstring,WCHAR	*idstring)=0;
	virtual	void	PadSymbol(float x, float y,int detail,tAircraftData*	pai_data,WCHAR	*altstring,WCHAR	*idstring)=0;

	void	CheckTooltip	(tAircraftData*	pai_data,double x,double y);
	void	PadOneAiSymbol	(tAircraftData*	pai_data,double	true_hdg,int act_index);
	void	DrawOneAiSymbol	(tAircraftData*	pai_data,double	true_hdg);
	void	DrawAiData		(double	true_hdg);
	ARGB	DimColour		(ARGB colour,int brightness);

public:	
	void	SetMouseCoordinates	(double	crs_x,double	crs_y);
	void	SetColours		(int brightness,int inact_colour,int strd_colour,int ta_colour,int ra_colour);
	void	SetDetails		(int inact_detail,int strd_detail,int ta_detail,int ra_detail,int padsymbol,char * fontname,int max_disp_act);
	void	ScaleScreen		(double size_x,double size_y,double scale_x,double scale_y,double centre_x,double centre_y,double range,BOOL nodeform,double	symbol_factor);
	void	SetCatRules		(double other_min_ralt,double other_above_alt,double other_below_alt,double proxy_range,double proxy_above_alt,double proxy_below_alt);	
	void	QueryAiData		(	UINT32 maxnum_act,FLOAT64 max_range_nm,
								FLOAT64 time_sep_ta,FLOAT64 time_sep_ra,
								FLOAT64 dist_sep_ta,FLOAT64 dist_sep_ra,
								FLOAT64 alt_sep_ta,FLOAT64 alt_sep_ra
								);
	bool	Draw			(HDC hdc,double	true_hdg,BOOL antialias);
	tAircraftData*	GetTooltipData	(void);
	tAircraftData*	GetAdvisoryData	(void);
	GdiPlusRadar	(void);
	virtual ~GdiPlusRadar	(void);
};

class RadarScreen	: public	GdiPlusRadar
{
	private:
		virtual	void	PadSymbol	(float x, float y,int detail,tAircraftData*	pai_data,WCHAR	*altstring,WCHAR	*idstring);
		virtual	void	DrawSymbol	(float x, float y,int detail,tAircraftData*	pai_data,FLOAT64	true_hdg,Color& color,WCHAR	*altstring,WCHAR	*idstring);
};

class TCASScreen	: public	GdiPlusRadar
{
	private:
		virtual	void	PadSymbol	(float x, float y,int detail,tAircraftData*	pai_data,WCHAR	*altstring,WCHAR	*idstring);
		virtual	void	DrawSymbol	(float x, float y,int detail,tAircraftData*	pai_data,FLOAT64	true_hdg,Color& color,WCHAR	*altstring,WCHAR	*idstring);
};


#endif//	GDIPLUSRADAR_H
