//XMLCCallbacks.cpp
//(c) 2006 Arne Bartels arne.bartels@t-online.de



//-----------------------------------------------------------------------------------
/*
* MyGaugeCDrawable implements the <CustomDraw...> method
* will be allocated at first acces to  <CustomDraw Name="TrafficRadar:TCAS" ... >
* or on resizing/2D<->3D switching
* one example per displayed gauge
*/
//-----------------------------------------------------------------------------------


FLAGS32	MyGaugeCDrawable::GetFlags(void)
{
	//called on startup
	return 		TAKES_DC;
}

void	MyGaugeCDrawable::Update(void)
{
	lookup_var(&m_tick18);
	if(m_radar==NULL)
		return;
	if(m_owner==NULL)
		return;
	if(m_tick18.var_old.n+TICK_MODULO>m_tick18.var_value.n)
		return;
	m_recalced	=	true;
	m_radar->QueryAiData((int)m_maxnum_act,m_max_range_nm,
						m_owner->GetTimeSepTA(),m_owner->GetTimeSepRA(),
						m_owner->GetDistanceSepTA(),m_owner->GetDistanceSepRA(),
						m_owner->GetAltitudeSepTA(),m_owner->GetAltitudeSepRA()
						);
	m_tick18.var_old	=	m_tick18.var_value;
}

void	MyGaugeCDrawable::Show(bool on)
{
	//necessary to define, but not needed in our example
}

bool	MyGaugeCDrawable::Draw(IGaugeCDrawableDrawParameters* pParameters, PIXPOINT size, HDC hdc, PIMAGE pImage)
{
	double		true_hdg		=	0,
				range			=	-1,
				centre_x		=	size.x/2,
				centre_y		=	size.y/2,
				symbol_size		=	1,

				other_min_ralt	=	CLEAR_OF_GROUND,
				other_above_alt	=	OTHER_DIFFALT,
				other_below_alt	=	OTHER_DIFFALT,
				proxy_range		=	PROXY_RANGE,
				proxy_above_alt	=	PROXY_DIFFALT,
				proxy_below_alt	=	PROXY_DIFFALT;
				
	int			brightness		=	255,
				other_detail	=	3,
				proxy_detail	=	3,
				ta_detail		=	3,
				ra_detail		=	3,
				max_disp_act	=	MAX_NUM_AIRCRAFT,
				
				other_colour	=	0x00F0F0F0,
				proxy_colour	=	0x0000FF00,
				ta_colour		=	0x00FFFF00,
				ra_colour		=	0x00FF0000;
	
	BOOL		antialias		=	TRUE,
				redraw			=	TRUE,
				fixedres		=	FALSE,
				padsymbol		=	FALSE;
	bool		result			=	FALSE;
	char		*fontname		=	NULL;



	redraw		=	m_recalced;


	//virtual methods, provided by the child classes
	//because different styles need different presets in ranges and colouring
	InitRanges(&other_min_ralt,&other_above_alt,&other_below_alt,&proxy_range,&proxy_above_alt,&proxy_below_alt);
	InitColours	(&other_colour,&proxy_colour,&ta_colour,&ra_colour);
	InitDetails(&other_detail,&proxy_detail,&ta_detail,&ra_detail);

	
	if(pParameters==NULL)
		return	false;
	centre_x	/=	pParameters->GetScaleX();
	centre_y	/=	pParameters->GetScaleY();

	if(m_owner==NULL)
		return	false;

	range		=	m_owner->GetRange();
	brightness	=	m_owner->GetBrightness();
	//	get the parameters from the <CustomDraw> statement in XML,
	//these have to be predeclared in CreateGaugeCDrawable

	//GetParameter doesn't change the value, if parameter isn't given in CustomDraw
	//so no further checks needed, simply try to get the values from the XML
	pParameters->GetParameter(DRAW_PROPERTY_RANGE,&range);
	pParameters->GetParameter(DRAW_PROPERTY_HEADING,&true_hdg);
	pParameters->GetParameter(DRAW_PROPERTY_BRIGHTNESS,&brightness);
	pParameters->GetParameter(DRAW_PROPERTY_OTHER_COL,&other_colour);
	pParameters->GetParameter(DRAW_PROPERTY_PROXY_COL,&proxy_colour);
	pParameters->GetParameter(DRAW_PROPERTY_TA_COL,&ta_colour);
	pParameters->GetParameter(DRAW_PROPERTY_RA_COL,&ra_colour);
	pParameters->GetParameter(DRAW_PROPERTY_ANTIALIAS,&antialias);

	pParameters->GetParameter(DRAW_PROPERTY_CENTRE_X,&centre_x);
	pParameters->GetParameter(DRAW_PROPERTY_CENTRE_Y,&centre_y);

	pParameters->GetParameter(DRAW_PROPERTY_SYMBOL_SIZE,&symbol_size);


	pParameters->GetParameter(DRAW_PROPERTY_INACT_DETAIL,&other_detail);
	pParameters->GetParameter(DRAW_PROPERTY_STRD_DETAIL,&proxy_detail);
	pParameters->GetParameter(DRAW_PROPERTY_TA_DETAIL,&ta_detail);
	pParameters->GetParameter(DRAW_PROPERTY_RA_DETAIL,&ra_detail);

	pParameters->GetParameter(DRAW_PROPERTY_FIXED_RES,&fixedres);
	pParameters->GetParameter(DRAW_PROPERTY_PADSYMBOL,&padsymbol);
	pParameters->GetParameter(DRAW_PROPERTY_SYMBOLFONT,(const char **)&fontname);
	

	pParameters->GetParameter(DRAW_PROPERTY_MAXRANGE,&m_max_range_nm);
	pParameters->GetParameter(DRAW_PROPERTY_MAXNUMACT,&m_maxnum_act);
	

	pParameters->GetParameter(DRAW_PROPERTY_OTHER_RALT,&other_min_ralt);
	pParameters->GetParameter(DRAW_PROPERTY_OTHER_ABOVE,&other_above_alt);
	pParameters->GetParameter(DRAW_PROPERTY_OTHER_BELOW,&other_below_alt);
	pParameters->GetParameter(DRAW_PROPERTY_PROXIMITY_RANGE,&proxy_range);
	pParameters->GetParameter(DRAW_PROPERTY_PROXIMITY_ABOVE,&proxy_above_alt);
	pParameters->GetParameter(DRAW_PROPERTY_PROXIMITY_BELOW,&proxy_below_alt);
	max_disp_act	=	ROUND(m_maxnum_act);
	pParameters->GetParameter(DRAW_PROPERTY_MAXDISPNUMACT,&max_disp_act);
	

	//redraw if brightness or range changed
	if(brightness	!=	m_oldbright)
		redraw		=	true;
	if(range		!=	m_oldrange)
		redraw		=	true;
	
	//remember present vals
	m_oldbright	=	brightness;	
	m_oldrange	=	range;

	//proceed only if class GdiPlusRadar exists
	if(m_radar==NULL)
		return	false;

	m_radar->SetMouseCoordinates(m_owner->GetXPosMouse(),m_owner->GetYPosMouse());
	m_radar->SetColours(brightness,other_colour,proxy_colour,ta_colour,ra_colour);
	m_radar->SetDetails(other_detail,proxy_detail,ta_detail,ra_detail,padsymbol,fontname,max_disp_act);
	m_radar->ScaleScreen(
		size.x,size.y,
		pParameters->GetScaleX(),pParameters->GetScaleY(),
		centre_x,centre_y,
		range,fixedres,
		symbol_size);
	m_radar->SetCatRules(other_min_ralt,other_above_alt,other_below_alt,proxy_range,proxy_above_alt,proxy_below_alt);




	//proceed only if drawing is necessary
	if(!redraw)
		return	false;

	m_recalced	=	false;
	
	result	=	m_radar->Draw(hdc,true_hdg,antialias);

	m_owner->SetTooltipData(m_radar->GetTooltipData());
	m_owner->SetAdvisoryData(m_radar->GetAdvisoryData());
	return	result;
}

bool	MyGaugeCDrawable::SetupDraw(PIXPOINT size, HDC hdc, PIMAGE pImage) 
{
	//necessary to define, but not needed in our example
	return	FALSE;
}
bool	MyGaugeCDrawable::GetDraw(IGaugeCDrawableDrawParameters* pParameters)
{
	//necessary to define, but not needed in our example
	return	FALSE;
}







//caution! defines several methods at once
DEFINE_PANEL_CALLBACK_REFCOUNT(MyGaugeCDrawable)
MyGaugeCDrawable::MyGaugeCDrawable	(MyGaugeCCallback	*owner)
{
	m_owner			=	owner;
	m_maxnum_act	=	MAX_NUM_AIRCRAFT;
	m_max_range_nm	=	MAX_RANGE_AIRCRAFT;	
	m_radar			=	NULL;

	m_oldbright		=	255;
	m_oldrange		=	0;	
	
	

	
	memset(&m_tick18,0,sizeof(m_tick18));
	m_tick18.id		=	TICK18;
	lookup_var(&m_tick18);
	m_tick18.var_old=m_tick18.var_value;


	//caution! initializes  several vars at once
	INIT_PANEL_CALLBACK_REFCOUNT(MyGaugeCDrawable);
}
MyGaugeCDrawable::~MyGaugeCDrawable	(void)
{
	SAFE_DELETE(m_radar)
}

//Child elements for Radar and TCAS 
//they differ in different presettings and drawing styles
//RadarCDrawable works as a radar sreen
//
RadarCDrawable::RadarCDrawable(MyGaugeCCallback	*owner)	:	MyGaugeCDrawable(owner)
{
	m_maxnum_act	=	100;
	m_max_range_nm	=	100;	
	m_radar			=	new	RadarScreen();
};
void	RadarCDrawable::InitRanges(double	*pother_min_ralt,
								   double	*pother_above_alt,
								   double	*pother_below_alt,
								   double	*pproxy_range,
								   double	*pproxy_above_alt,
								   double	*proxy_below_alt)
{
	/*
	 These variables determine the categories OTHER and PROXIMITY
	 here: all aircraft are shown (very big above_alt and below_alt) 
	       even the ones on ground (other_min_ralt=0).
		  all aircraft exept those on ground (OTHER) are shown as PROXIMITY,
		  by using the same (or bigger) proxy_range,proxy_above_alt,proxy_below_alt
		  as for OTHER
	*/
	if(pother_min_ralt)		*pother_min_ralt	=	0;
	if(pother_above_alt)	*pother_above_alt	=	100000;
	if(pother_below_alt)	*pother_below_alt	=	100000;
	if(pproxy_range)		*pproxy_range		=	m_max_range_nm;
	if(pproxy_above_alt)	*pproxy_above_alt	=	100000;
	if(proxy_below_alt)		*proxy_below_alt	=	100000;
}
void	RadarCDrawable::InitColours(int	*pother_colour,int	*pproxy_colour,int	*pta_colour,int	*pra_colour)
{
	if(pother_colour)	*pother_colour	=	0x00F0F0F0;//on ground: white
	if(pproxy_colour)	*pproxy_colour	=	0x0000FF00;//in air: green
	if(pta_colour)		*pta_colour		=	0x00FFFF00;//TA yellow
	if(pra_colour)		*pra_colour		=	0x00FF0000;//RA red
}
void	RadarCDrawable::InitDetails(int	*pother_detail,int	*pproxy_detail,int	*pta_detail,int	*pra_detail)
{
	if(pother_detail)	*pother_detail	=	DETAIL_SYMBOL_OUTLINE|DETAIL_ALTITUDE;
	if(pproxy_detail)	*pproxy_detail	=	DETAIL_SYMBOL_OUTLINE|DETAIL_ALTITUDE;
	if(pta_detail)		*pta_detail		=	DETAIL_SYMBOL_OUTLINE|DETAIL_ALTITUDE;
	if(pra_detail)		*pra_detail		=	DETAIL_SYMBOL_OUTLINE|DETAIL_ALTITUDE;
}

//TCASCDrawable works as a TCAS system

TCASCDrawable::TCASCDrawable(MyGaugeCCallback	*owner)	:	MyGaugeCDrawable(owner)
{
	m_maxnum_act	=	100;
	m_max_range_nm	=	40;	
	m_radar			=	new	TCASScreen();
};
void	TCASCDrawable::InitRanges(double	*pother_min_ralt,
								   double	*pother_above_alt,
								   double	*pother_below_alt,
								   double	*pproxy_range,
								   double	*pproxy_above_alt,
								   double	*proxy_below_alt)
{
	/*
	 These variables determine the categories OTHER and PROXIMITY
	 here: a display style like a TCAs is used,
	        - no aircraft on ground (other_min_ralt>0) 
	        - OTHER within +-2700ft
			- PROXIMITY within +-1200ft and 6nm
	*/
	if(pother_min_ralt)		*pother_min_ralt	=	CLEAR_OF_GROUND;
	if(pother_above_alt)	*pother_above_alt	=	2700;
	if(pother_below_alt)	*pother_below_alt	=	2700;
	if(pproxy_range)		*pproxy_range		=	6;
	if(pproxy_above_alt)	*pproxy_above_alt	=	1200;
	if(proxy_below_alt)		*proxy_below_alt	=	1200;
}
void	TCASCDrawable::InitColours(int	*pother_colour,int	*pproxy_colour,int	*pta_colour,int	*pra_colour)
{
	if(pother_colour)	*pother_colour	=	0x00F0F0F0;//other aircraft: hollow white diamond (some use blue here)
	if(pproxy_colour)	*pproxy_colour	=	0x00F0F0F0;//proximity:		 filled white diamond (some use blue here)
	if(pta_colour)		*pta_colour		=	0x00FFFF00;//TA              yellow filled circle
	if(pra_colour)		*pra_colour		=	0x00FF0000;//RA              red filled square
}

void	TCASCDrawable::InitDetails(int	*pother_detail,int	*pproxy_detail,int	*pta_detail,int	*pra_detail)
{
	if(pother_detail)	*pother_detail	=	DETAIL_SYMBOL_OUTLINE|DETAIL_ALTITUDE;						//other aircraft:	hollow white diamond 
	if(pproxy_detail)	*pproxy_detail	=	DETAIL_SYMBOL_OUTLINE|DETAIL_SYMBOL_FILLED|DETAIL_ALTITUDE;	//proximity:		filled white diamond 
	if(pta_detail)		*pta_detail		=	DETAIL_SYMBOL_OUTLINE|DETAIL_SYMBOL_FILLED|DETAIL_ALTITUDE;	//TA				yellow filled circle
	if(pra_detail)		*pra_detail		=	DETAIL_SYMBOL_OUTLINE|DETAIL_SYMBOL_FILLED|DETAIL_ALTITUDE;	//RA				red filled square
}



//-----------------------------------------------------------------------------------
/*
* MyGaugeCCallback
* will be allocated at first acces to any C:TrafficRadar:... variable or at resize/2D<->3D switching
* one example per displayed gauge
*/
//-----------------------------------------------------------------------------------
int		MyGaugeCCallback::GetBrightness(void)
{
	return	m_brightness;
};
void	MyGaugeCCallback::SetBrightness(int value)
{
	m_brightness = value;
};
	
double	MyGaugeCCallback::GetRange(void)
{
	return	m_range;
};
void	MyGaugeCCallback::SetRange(double value)
{
	m_range = value;
};
double	MyGaugeCCallback::GetXPosMouse(void)
{
	return	m_xposmouse;
};
void	MyGaugeCCallback::SetXPosMouse(double xposmouse)
{
	m_xposmouse	=	xposmouse;
};
double	MyGaugeCCallback::GetYPosMouse(void)
{
	return m_yposmouse;
};
void	MyGaugeCCallback::SetYPosMouse(double yposmouse)
{
	m_yposmouse	=	yposmouse;
};



char*	MyGaugeCCallback::GetTooltip(void)
{
	return	m_tooltip;
};
void	MyGaugeCCallback::SetTooltipData		(tAircraftData* value)
{
	if(value==NULL)
		return;
	memcpy(&m_tooltip_data,value,sizeof(tAircraftData));
	memset(m_tooltip,0,sizeof(m_tooltip));
	if(strlen(m_tooltip_data.atc_id))
		sprintf_s(m_tooltip,"ID: %s ALT: %dft GS: %.0fkt TRK: %.0f DST: %.1fnm",
			m_tooltip_data.atc_id,
			ROUND(m_tooltip_data.dlt_alt/100)*100,
			m_tooltip_data.grnd_spd,
			m_tooltip_data.trck_angl,
			m_tooltip_data.dist);
}

void	MyGaugeCCallback::SetAdvisoryData		(tAircraftData* value)
{
	if(value==NULL)
		return;
	memcpy(&m_advisory_data,value,sizeof(tAircraftData));
};


//Time Separation-------------------------------
double		MyGaugeCCallback::GetTimeSepTA(void)
{
	return	m_time_sep_ta;
};
void		MyGaugeCCallback::SetTimeSepTA(double value)
{
	m_time_sep_ta	=	value;
};

double		MyGaugeCCallback::GetTimeSepRA(void)
{
	return	m_time_sep_ra;
};
void		MyGaugeCCallback::SetTimeSepRA(double value)
{
	m_time_sep_ra	=	value;
};
//----------------------------------------------



//Distance Separation-------------------------------
double		MyGaugeCCallback::GetDistanceSepTA(void)
{
	return	m_distance_sep_ta;
};
void		MyGaugeCCallback::SetDistanceSepTA(double value)
{
	m_distance_sep_ta	=	value;
};
double		MyGaugeCCallback::GetDistanceSepRA(void)
{
	return	m_distance_sep_ra;
};
void		MyGaugeCCallback::SetDistanceSepRA(double value)
{
	m_distance_sep_ra	=	value;
};
//----------------------------------------------


//Altitude Separation-------------------------------
double		MyGaugeCCallback::GetAltitudeSepTA(void)
{
	return	m_altitude_sep_ta;
};
void		MyGaugeCCallback::SetAltitudeSepTA(double value)
{
		m_altitude_sep_ta	=	value;
};
double		MyGaugeCCallback::GetAltitudeSepRA(void)
{
	return	m_altitude_sep_ra;
};
void		MyGaugeCCallback::SetAltitudeSepRA(double value)
{
		m_altitude_sep_ra	=	value;
};
//----------------------------------------------


IGaugeCCallback*	MyGaugeCCallback::QueryInterface(PCSTRINGZ pszInterface)
{
	//necessary to define, but not needed in our example
	if(pszInterface	==	NULL)
		return	NULL;

	if(_stricmp(pszInterface,INTERFACE_NAME) == 0)
		return	this;

	return	NULL;
}

//will be called 18 times per sec just after MyAircraftCCallback::Update
void	MyGaugeCCallback::Update(void)
{
	//necessary to define, but not needed in our example
}

bool	MyGaugeCCallback::GetPropertyValue(SINT32 id, FLOAT64* pValue)
{
	//mapping of property ids to actual float values
	if(pValue	==	NULL)
		return	FALSE;
	
	switch(id)
	{
		case	PROPERTY_RANGE:
			*pValue	=	GetRange();
			break;
		case	PROPERTY_BRIGHT:
			*pValue	=	GetBrightness();
			break;
		case	PROPERTY_XMOUSE:
			*pValue	=	GetXPosMouse();
			break;
		case	PROPERTY_YMOUSE:
			*pValue	=	GetYPosMouse();
			break;
		case	PROPERTY_ADVISORY:
			*pValue	=	m_advisory_data.tcas_state;
			break;
		case	PROPERTY_TIM_SEP_TA:
			*pValue	=	GetTimeSepTA();
			break;
		case	PROPERTY_TIM_SEP_RA:
			*pValue	=	GetTimeSepRA();
			break;
		case	PROPERTY_DST_SEP_TA:
			*pValue	=	GetDistanceSepTA();
			break;
		case	PROPERTY_DST_SEP_RA:
			*pValue	=	GetDistanceSepRA();
			break;
		case	PROPERTY_ALT_SEP_TA:
			*pValue	=	GetAltitudeSepTA();
			break;
		case	PROPERTY_ALT_SEP_RA:
			*pValue	=	GetAltitudeSepRA();
			break;

		case	PROPERTY_TTIP_ALT:
			*pValue	=	m_tooltip_data.alt;
			break;
		case	PROPERTY_TTIP_DIST:
			*pValue	=	m_tooltip_data.dist;
			break;
		case	PROPERTY_TTIP_BRG:
			*pValue	=	m_tooltip_data.brg;
			break;
		case	PROPERTY_TTIP_DELTA_ALT:
			*pValue	=	m_tooltip_data.dlt_alt;
			break;
		case	PROPERTY_TTIP_TRACK_ANGLE:
			*pValue	=	m_tooltip_data.trck_angl;
			break;
		case	PROPERTY_TTIP_GROUNDSPEED:
			*pValue	=	m_tooltip_data.grnd_spd;
			break;

		case	PROPERTY_ADV_ALT:
			*pValue	=	m_advisory_data.alt;
			break;
		case	PROPERTY_ADV_DIST:
			*pValue	=	m_advisory_data.dist;
			break;
		case	PROPERTY_ADV_BRG:
			*pValue	=	m_advisory_data.brg;
			break;
		case	PROPERTY_ADV_DELTA_ALT:
			*pValue	=	m_advisory_data.dlt_alt;
			break;
		case	PROPERTY_ADV_TRACK_ANGLE:
			*pValue	=	m_advisory_data.trck_angl;
			break;
		case	PROPERTY_ADV_GROUNDSPEED:
			*pValue	=	m_advisory_data.grnd_spd;
			break;

		case	PROPERTY_TTIP_RADALT:
			*pValue	=	m_tooltip_data.alt_agl;
			break;
		case	PROPERTY_ADV_RADALT:
			*pValue	=	m_advisory_data.alt_agl;
			break;
		default:
			return	FALSE;
	}
	return	TRUE;
}

bool	MyGaugeCCallback::GetPropertyValue(SINT32 id, PCSTRINGZ* pszValue)
{
	//mapping of property ids to actual string value
	if(pszValue	==	NULL)
		return	FALSE;


	switch(id)
	{
		case	PROPERTY_TTIP:
			*pszValue	=	GetTooltip();
			break;
		case	PROPERTY_TTIP_ID:
			*pszValue	=	m_tooltip_data.atc_id;
			break;
		case	PROPERTY_ADV_ID:
			*pszValue	=	m_advisory_data.atc_id;
			break;
		default:
			return	FALSE;
	}
	return	TRUE;
}

bool	MyGaugeCCallback::SetPropertyValue(SINT32 id, FLOAT64 value)
{

	switch(id)
	{
		case	PROPERTY_RANGE:
			SetRange(value);
			break;
		case	PROPERTY_BRIGHT:
			SetBrightness((int)value);
			break;
		case	PROPERTY_XMOUSE:
			SetXPosMouse(value);
			break;
		case	PROPERTY_YMOUSE:
			SetYPosMouse(value);
			break;
		case	PROPERTY_TIM_SEP_TA:
			SetTimeSepTA(value);
			break;
		case	PROPERTY_TIM_SEP_RA:
			SetTimeSepRA(value);
			break;
		case	PROPERTY_DST_SEP_TA:
			SetDistanceSepTA(value);
			break;
		case	PROPERTY_DST_SEP_RA:
			SetDistanceSepRA(value);
			break;
		case	PROPERTY_ALT_SEP_TA:
			SetAltitudeSepTA(value);
			break;
		case	PROPERTY_ALT_SEP_RA:
			SetAltitudeSepRA(value);
			break;
		default:
			return	FALSE;
	}
	return	TRUE;	
}



bool	MyGaugeCCallback::SetPropertyValue(SINT32 id, PCSTRINGZ szValue)
{
	//necessary to define, but not needed in our example
	//string is read-only
	return	FALSE;
}

IGaugeCDrawable*	MyGaugeCCallback::CreateGaugeCDrawable	(SINT32 id, const IGaugeCDrawableCreateParameters* pParameters) 
{

	if(pParameters!=NULL)
	{
	//associate property IDs to vars for later use in MyGaugeCDrawable::Draw();
	//vars in the form of:


		pParameters->SetParameterId (DRAW_NAME_RANGE,			DRAW_PROPERTY_RANGE,		pParameters->PARAMETER_TYPE_FLOAT);
		pParameters->SetParameterId (DRAW_NAME_HEADING,			DRAW_PROPERTY_HEADING,		pParameters->PARAMETER_TYPE_FLOAT);
		pParameters->SetParameterId (DRAW_NAME_BRIGHTNESS,		DRAW_PROPERTY_BRIGHTNESS,	pParameters->PARAMETER_TYPE_INT);
		
		pParameters->SetParameterId (DRAW_NAME_OTHER_COL,		DRAW_PROPERTY_OTHER_COL,	pParameters->PARAMETER_TYPE_INT);
		pParameters->SetParameterId (DRAW_NAME_PROXY_COL,		DRAW_PROPERTY_PROXY_COL,	pParameters->PARAMETER_TYPE_INT);
		pParameters->SetParameterId (DRAW_NAME_TA_COL,			DRAW_PROPERTY_TA_COL,		pParameters->PARAMETER_TYPE_INT);
		pParameters->SetParameterId (DRAW_NAME_RA_COL,			DRAW_PROPERTY_RA_COL,		pParameters->PARAMETER_TYPE_INT);
		pParameters->SetParameterId (DRAW_NAME_ANTIALIAS,		DRAW_PROPERTY_ANTIALIAS,	pParameters->PARAMETER_TYPE_INT);

		pParameters->SetParameterId (DRAW_NAME_MAXRANGE,		DRAW_PROPERTY_MAXRANGE,		pParameters->PARAMETER_TYPE_FLOAT);
		pParameters->SetParameterId (DRAW_NAME_MAXNUMACT,		DRAW_PROPERTY_MAXNUMACT,	pParameters->PARAMETER_TYPE_FLOAT);
	
		pParameters->SetParameterId (DRAW_NAME_CENTRE_X,		DRAW_PROPERTY_CENTRE_X,		pParameters->PARAMETER_TYPE_FLOAT);
		pParameters->SetParameterId (DRAW_NAME_CENTRE_Y,		DRAW_PROPERTY_CENTRE_Y,		pParameters->PARAMETER_TYPE_FLOAT);

		pParameters->SetParameterId (DRAW_NAME_SYMBOL_SIZE,		DRAW_PROPERTY_SYMBOL_SIZE,	pParameters->PARAMETER_TYPE_FLOAT);
		pParameters->SetParameterId (DRAW_NAME_INACT_DETAIL,	DRAW_PROPERTY_INACT_DETAIL,	pParameters->PARAMETER_TYPE_INT);
		pParameters->SetParameterId (DRAW_NAME_STRD_DETAIL,		DRAW_PROPERTY_STRD_DETAIL,	pParameters->PARAMETER_TYPE_INT);
		pParameters->SetParameterId (DRAW_NAME_TA_DETAIL,		DRAW_PROPERTY_TA_DETAIL,	pParameters->PARAMETER_TYPE_INT);
		pParameters->SetParameterId (DRAW_NAME_RA_DETAIL,		DRAW_PROPERTY_RA_DETAIL,	pParameters->PARAMETER_TYPE_INT);

		pParameters->SetParameterId (DRAW_NAME_FIXED_RES,		DRAW_PROPERTY_FIXED_RES,	pParameters->PARAMETER_TYPE_INT);
		pParameters->SetParameterId (DRAW_NAME_PADSYMBOL,		DRAW_PROPERTY_PADSYMBOL,	pParameters->PARAMETER_TYPE_INT);
		pParameters->SetParameterId (DRAW_NAME_SYMBOLFONT,		DRAW_PROPERTY_SYMBOLFONT,	pParameters->PARAMETER_TYPE_STRING);


		pParameters->SetParameterId (DRAW_NAME_OTHER_RALT,		DRAW_PROPERTY_OTHER_RALT,	pParameters->PARAMETER_TYPE_FLOAT);
		pParameters->SetParameterId (DRAW_NAME_OTHER_ABOVE,		DRAW_PROPERTY_OTHER_ABOVE,	pParameters->PARAMETER_TYPE_FLOAT);
		pParameters->SetParameterId (DRAW_NAME_OTHER_BELOW,		DRAW_PROPERTY_OTHER_BELOW,	pParameters->PARAMETER_TYPE_FLOAT);
		pParameters->SetParameterId (DRAW_NAME_PROXIMITY_RANGE,	DRAW_PROPERTY_PROXIMITY_RANGE,	pParameters->PARAMETER_TYPE_FLOAT);
		pParameters->SetParameterId (DRAW_NAME_PROXIMITY_ABOVE,	DRAW_PROPERTY_PROXIMITY_ABOVE,	pParameters->PARAMETER_TYPE_FLOAT);
		pParameters->SetParameterId (DRAW_NAME_PROXIMITY_BELOW,	DRAW_PROPERTY_PROXIMITY_BELOW,	pParameters->PARAMETER_TYPE_FLOAT);

		pParameters->SetParameterId (DRAW_NAME_MAXDISPNUMACT,	DRAW_PROPERTY_MAXDISPNUMACT,	pParameters->PARAMETER_TYPE_INT);


		//backward compability
		pParameters->SetParameterId (OLDDRAW_NAME_INACT_COL,	DRAW_PROPERTY_OTHER_COL,	pParameters->PARAMETER_TYPE_INT);
		pParameters->SetParameterId (OLDDRAW_NAME_STRD_COL,		DRAW_PROPERTY_PROXY_COL,	pParameters->PARAMETER_TYPE_INT);

	
	}
	if(id==PROPERTY_RADAR)
		return	new RadarCDrawable(this);
	if(id==PROPERTY_TCAS)
		return	new TCASCDrawable(this);
	return	NULL;
}



//caution! defines several methods at once
DEFINE_PANEL_CALLBACK_REFCOUNT(MyGaugeCCallback)

MyGaugeCCallback::MyGaugeCCallback(MyAircraftCCallback *acft_owner)
{
	//preset standard values
	memset(&m_tooltip_data,0,sizeof(m_tooltip_data));
	memset(m_tooltip,0,sizeof(m_tooltip));
	memset(&m_advisory_data,0,sizeof(m_advisory_data));

	m_range			=	INITIAL_RANGE;
	m_brightness	=	255;
	m_xposmouse		=	0;
	m_yposmouse		=	0;
	m_time_sep_ta		=	35;
	m_time_sep_ra		=	20;
	m_distance_sep_ta	=	1;
	m_distance_sep_ra	=	0.1;
	m_altitude_sep_ta	=	1200;
	m_altitude_sep_ra	=	300;

	
	
	
	//caution! initializes  several vars at once
	INIT_PANEL_CALLBACK_REFCOUNT(MyGaugeCCallback);

}

MyGaugeCCallback::~MyGaugeCCallback	(void)
{
}


//-----------------------------------------------------------------------------------
/*
* MyAircraftCCallback
* will be allocated at first acces to any C:TrafficRadar:... variable
* released on aircraft switching
*/
//-----------------------------------------------------------------------------------
//not called, but better coded, then sorry later
//possibly callable from "panel_get_aircraft_c_callback"
IAircraftCCallback* MyAircraftCCallback::QueryInterface(PCSTRINGZ pszInterface)
{
	if(pszInterface	==	NULL)
		return	NULL;

	if(_stricmp(pszInterface,INTERFACE_NAME) == 0)
		return	this;

	return	NULL;
}

IGaugeCCallback*	MyAircraftCCallback::CreateGaugeCCallback(void)
{
	return	new	MyGaugeCCallback(this);
}

//will be called 18 times per sec just before MyGaugeCCallback::Update, 
//but in difference this method aircraft specific and called only once in a refesh cycle
//so a good place for autopilot code and similar
//not used at all in this example 
void	MyAircraftCCallback::Update(void)
{
}

//caution! defines several methods at once
DEFINE_PANEL_CALLBACK_REFCOUNT(MyAircraftCCallback)

MyAircraftCCallback::MyAircraftCCallback(void)
{
	//caution! initializes  several vars at once
	INIT_PANEL_CALLBACK_REFCOUNT(MyAircraftCCallback);
}

MyAircraftCCallback::~MyAircraftCCallback(void)
{
}



//-----------------------------------------------------------------------------------
/*
* IPanelCCallback
* will be allocated at FS startup
*/
//-----------------------------------------------------------------------------------
//possibly callable from "panel_get_registered_c_callback"

IPanelCCallback*	MyPanelCCallback::QueryInterface(PCSTRINGZ pszInterface)
{
	if(pszInterface	==	NULL)
		return	NULL;

	if(_stricmp(pszInterface,INTERFACE_NAME) == 0)
		return	this;

	return	NULL;
}

//called at module loadtime
UINT32	MyPanelCCallback::GetVersion(void)
{
	return	VERSION_MAJOR*0x10000+VERSION_MINOR*0x100+VERSION_BUILD;
}

//allocates the related MyAircraftCCallback object
//will be allocated at first acces to any C:TrafficRadar:... variable
IAircraftCCallback*	MyPanelCCallback::CreateAircraftCCallback(UINT32 ContainerID)
{
	return	new	MyAircraftCCallback();
}

//map property IDs to names used in XML gauges, other callbacks use these property IDs
//called on loading of xmlgauges with references to "C:TrafficRadar:...
bool	MyPanelCCallback::ConvertStringToProperty(PCSTRINGZ keyword, SINT32* pID)
{
	if(keyword	==	NULL	||	pID	==	NULL)
		return	FALSE;

	if(m_oldinterface)
	{
		//backward compability
		if(_stricmp(OLDNAME_DRAW,keyword)	==	0)
		{
			*pID	=	PROPERTY_RADAR;
			return	TRUE;
		}

	}
	else
	{
		if(_stricmp(NAME_RADAR,keyword)	==	0)
		{
			*pID	=	PROPERTY_RADAR;
			return	TRUE;
		}

		if(_stricmp(NAME_TCAS,keyword)	==	0)
		{
			*pID	=	PROPERTY_TCAS;
			return	TRUE;
		}
	}


	
	
	if((_stricmp(NAME_RANGE,keyword)	==	0)||(_stricmp(OLDNAME_RANGE,keyword)	==	0))
	{
		*pID	=	PROPERTY_RANGE;
		return	TRUE;
	}
	if((_stricmp(NAME_BRIGHT,keyword)	==	0)||(_stricmp(OLDNAME_BRIGHT,keyword)	==	0))
	{
		*pID	=	PROPERTY_BRIGHT;
		return	TRUE;
	}
	if(_stricmp(NAME_YMOUSE,keyword)	==	0)
	{
		*pID	=	PROPERTY_YMOUSE;
		return	TRUE;
	}
	if(_stricmp(NAME_XMOUSE,keyword)	==	0)
	{
		*pID	=	PROPERTY_XMOUSE;
		return	TRUE;
	}
	if(_stricmp(NAME_TTIP,keyword)	==	0)
	{
		*pID	=	PROPERTY_TTIP;
		return	TRUE;
	}
	if(_stricmp(NAME_ADVISORY,keyword)	==	0)
	{
		*pID	=	PROPERTY_ADVISORY;
		return	TRUE;
	}
	if(_stricmp(NAME_TIM_SEP_TA,keyword)	==	0)
	{
		*pID	=	PROPERTY_TIM_SEP_TA;
		return	TRUE;
	}
	if(_stricmp(NAME_TIM_SEP_RA,keyword)	==	0)
	{
		*pID	=	PROPERTY_TIM_SEP_RA;
		return	TRUE;
	}
	if(_stricmp(NAME_DST_SEP_TA,keyword)	==	0)
	{
		*pID	=	PROPERTY_DST_SEP_TA;
		return	TRUE;
	}
	if(_stricmp(NAME_DST_SEP_RA,keyword)	==	0)
	{
		*pID	=	PROPERTY_DST_SEP_RA;
		return	TRUE;
	}
	if(_stricmp(NAME_ALT_SEP_TA,keyword)	==	0)
	{
		*pID	=	PROPERTY_ALT_SEP_TA;
		return	TRUE;
	}
	if(_stricmp(NAME_ALT_SEP_RA,keyword)	==	0)
	{
		*pID	=	PROPERTY_ALT_SEP_RA;
		return	TRUE;
	}

	if(_stricmp(NAME_TTIP_ID,keyword)	==	0)
	{
		*pID	=	PROPERTY_TTIP_ID;
		return	TRUE;
	}
	if(_stricmp(NAME_TTIP_ALT,keyword)	==	0)
	{
		*pID	=	PROPERTY_TTIP_ALT;
		return	TRUE;
	}
	if(_stricmp(NAME_TTIP_DIST,keyword)	==	0)
	{
		*pID	=	PROPERTY_TTIP_DIST;
		return	TRUE;
	}
	if(_stricmp(NAME_TTIP_BRG,keyword)	==	0)
	{
		*pID	=	PROPERTY_TTIP_BRG;
		return	TRUE;
	}
	if(_stricmp(NAME_TTIP_DELTA_ALT,keyword)	==	0)
	{
		*pID	=	PROPERTY_TTIP_DELTA_ALT;
		return	TRUE;
	}
	if(_stricmp(NAME_TTIP_TRACK_ANGLE,keyword)	==	0)
	{
		*pID	=	PROPERTY_TTIP_TRACK_ANGLE;
		return	TRUE;
	}
	if(_stricmp(NAME_TTIP_GROUNDSPEED,keyword)	==	0)
	{
		*pID	=	PROPERTY_TTIP_GROUNDSPEED;
		return	TRUE;
	}

	if(_stricmp(NAME_ADV_ID,keyword)	==	0)
	{
		*pID	=	PROPERTY_ADV_ID;
		return	TRUE;
	}
	if(_stricmp(NAME_ADV_ALT,keyword)	==	0)
	{
		*pID	=	PROPERTY_ADV_ALT;
		return	TRUE;
	}
	if(_stricmp(NAME_ADV_DIST,keyword)	==	0)
	{
		*pID	=	PROPERTY_ADV_DIST;
		return	TRUE;
	}
	if(_stricmp(NAME_ADV_BRG,keyword)	==	0)
	{
		*pID	=	PROPERTY_ADV_BRG;
		return	TRUE;
	}
	if(_stricmp(NAME_ADV_DELTA_ALT,keyword)	==	0)
	{
		*pID	=	PROPERTY_ADV_DELTA_ALT;
		return	TRUE;
	}
	if(_stricmp(NAME_ADV_TRACK_ANGLE,keyword)	==	0)
	{
		*pID	=	PROPERTY_ADV_TRACK_ANGLE;
		return	TRUE;
	}
	if(_stricmp(NAME_ADV_GROUNDSPEED,keyword)	==	0)
	{
		*pID	=	PROPERTY_ADV_GROUNDSPEED;
		return	TRUE;
	}

	if(_stricmp(NAME_TTIP_RADALT,keyword)	==	0)
	{
		*pID	=	PROPERTY_TTIP_RADALT;
		return	TRUE;
	}
	if(_stricmp(NAME_ADV_RADALT,keyword)	==	0)
	{
		*pID	=	PROPERTY_ADV_RADALT;
		return	TRUE;
	}
	return	FALSE;
}

//map names used in XML gauges to property IDs
bool	MyPanelCCallback::ConvertPropertyToString(SINT32 id, PPCSTRINGZ pKeyword)
{
	if(pKeyword	==	NULL)
		return	FALSE;

	switch(id)
	{
		//OK, using he length of the source string is not what strcpy_s was build for,
		//but where should I get the allowed length of pKeyword??? It is nowhere documented.
		case	PROPERTY_RADAR:
			strcpy_s((char *)(*pKeyword),strlen(NAME_TCAS),NAME_RADAR);
			break;
		case	PROPERTY_TCAS:
			strcpy_s((char *)(*pKeyword),strlen(NAME_TCAS),NAME_TCAS);
			break;
		case	PROPERTY_RANGE:
			strcpy_s((char *)(*pKeyword),strlen(NAME_RANGE),NAME_RANGE);
			break;
		case	PROPERTY_BRIGHT:
			strcpy_s((char *)(*pKeyword),strlen(NAME_BRIGHT),NAME_BRIGHT);
			break;
		case	PROPERTY_XMOUSE:
			strcpy_s((char *)(*pKeyword),strlen(NAME_XMOUSE),NAME_XMOUSE);
			break;
		case	PROPERTY_YMOUSE:
			strcpy_s((char *)(*pKeyword),strlen(NAME_YMOUSE),NAME_YMOUSE);
			break;
		case	PROPERTY_TTIP:
			strcpy_s((char *)(*pKeyword),strlen(NAME_TTIP),NAME_TTIP);
			break;
		case	PROPERTY_ADVISORY:
			strcpy_s((char *)(*pKeyword),strlen(NAME_ADVISORY),NAME_ADVISORY);
			break;
		case	PROPERTY_TIM_SEP_TA:
			strcpy_s((char *)(*pKeyword),strlen(NAME_TIM_SEP_TA),NAME_TIM_SEP_TA);
			break;
		case	PROPERTY_TIM_SEP_RA:
			strcpy_s((char *)(*pKeyword),strlen(NAME_TIM_SEP_RA),NAME_TIM_SEP_RA);
			break;
		case	PROPERTY_DST_SEP_TA:
			strcpy_s((char *)(*pKeyword),strlen(NAME_DST_SEP_TA),NAME_DST_SEP_TA);
			break;
		case	PROPERTY_DST_SEP_RA:
			strcpy_s((char *)(*pKeyword),strlen(NAME_DST_SEP_RA),NAME_DST_SEP_RA);
			break;
		case	PROPERTY_ALT_SEP_TA:
			strcpy_s((char *)(*pKeyword),strlen(NAME_ALT_SEP_TA),NAME_ALT_SEP_TA);
			break;
		case	PROPERTY_ALT_SEP_RA:
			strcpy_s((char *)(*pKeyword),strlen(NAME_ALT_SEP_RA),NAME_ALT_SEP_RA);
			break;

		case	PROPERTY_TTIP_ID:
			strcpy_s((char *)(*pKeyword),strlen(NAME_TTIP_ID),NAME_TTIP_ID);
			break;
		case	PROPERTY_TTIP_ALT:
			strcpy_s((char *)(*pKeyword),strlen(NAME_TTIP_ALT),NAME_TTIP_ALT);
			break;
		case	PROPERTY_TTIP_DIST:
			strcpy_s((char *)(*pKeyword),strlen(NAME_TTIP_DIST),NAME_TTIP_DIST);
			break;
		case	PROPERTY_TTIP_BRG:
			strcpy_s((char *)(*pKeyword),strlen(NAME_TTIP_BRG),NAME_TTIP_BRG);
			break;
		case	PROPERTY_TTIP_DELTA_ALT:
			strcpy_s((char *)(*pKeyword),strlen(NAME_TTIP_DELTA_ALT),NAME_TTIP_DELTA_ALT);
			break;
		case	PROPERTY_TTIP_TRACK_ANGLE:
			strcpy_s((char *)(*pKeyword),strlen(NAME_TTIP_TRACK_ANGLE),NAME_TTIP_TRACK_ANGLE);
			break;
		case	PROPERTY_TTIP_GROUNDSPEED:
			strcpy_s((char *)(*pKeyword),strlen(NAME_TTIP_GROUNDSPEED),NAME_TTIP_GROUNDSPEED);
			break;

		case	PROPERTY_ADV_ID:
			strcpy_s((char *)(*pKeyword),strlen(NAME_ADV_ID),NAME_ADV_ID);
			break;
		case	PROPERTY_ADV_ALT:
			strcpy_s((char *)(*pKeyword),strlen(NAME_ADV_ALT),NAME_ADV_ALT);
			break;
		case	PROPERTY_ADV_DIST:
			strcpy_s((char *)(*pKeyword),strlen(NAME_ADV_DIST),NAME_ADV_DIST);
			break;
		case	PROPERTY_ADV_BRG:
			strcpy_s((char *)(*pKeyword),strlen(NAME_ADV_BRG),NAME_ADV_BRG);
			break;
		case	PROPERTY_ADV_DELTA_ALT:
			strcpy_s((char *)(*pKeyword),strlen(NAME_ADV_DELTA_ALT),NAME_ADV_DELTA_ALT);
			break;
		case	PROPERTY_ADV_TRACK_ANGLE:
			strcpy_s((char *)(*pKeyword),strlen(NAME_ADV_TRACK_ANGLE),NAME_ADV_TRACK_ANGLE);
			break;
		case	PROPERTY_ADV_GROUNDSPEED:
			strcpy_s((char *)(*pKeyword),strlen(NAME_ADV_GROUNDSPEED),NAME_ADV_GROUNDSPEED);
			break;

		case	PROPERTY_TTIP_RADALT:
			strcpy_s((char *)(*pKeyword),strlen(NAME_TTIP_RADALT),NAME_TTIP_RADALT);
			break;
		case	PROPERTY_ADV_RADALT:
			strcpy_s((char *)(*pKeyword),strlen(NAME_ADV_RADALT),NAME_ADV_RADALT);
			break;
		default:
			return	FALSE;
	}
	return	TRUE;	
}

//map units enums to property ids
//use UNITS_UNKNOWN (aka "number") if not string
bool	MyPanelCCallback::GetPropertyUnits(SINT32 id, ENUM* pEnum)
{
	if(pEnum	==	NULL)
		return	FALSE;

	switch(id)
	{
		case	PROPERTY_RADAR:
		case	PROPERTY_TCAS:
		case	PROPERTY_RANGE:
		case	PROPERTY_BRIGHT:
		case	PROPERTY_XMOUSE:
		case	PROPERTY_YMOUSE:
		case	PROPERTY_ADVISORY:
		case	PROPERTY_TIM_SEP_TA:
		case	PROPERTY_TIM_SEP_RA:
		case	PROPERTY_DST_SEP_TA:
		case	PROPERTY_DST_SEP_RA:
		case	PROPERTY_ALT_SEP_TA:
		case	PROPERTY_ALT_SEP_RA:
		case	PROPERTY_TTIP_ALT:
		case	PROPERTY_TTIP_DIST:
		case	PROPERTY_TTIP_BRG:
		case	PROPERTY_TTIP_DELTA_ALT:
		case	PROPERTY_TTIP_TRACK_ANGLE:
		case	PROPERTY_TTIP_GROUNDSPEED:
		case	PROPERTY_ADV_ALT:
		case	PROPERTY_ADV_DIST:
		case	PROPERTY_ADV_BRG:
		case	PROPERTY_ADV_DELTA_ALT:
		case	PROPERTY_ADV_TRACK_ANGLE:
		case	PROPERTY_ADV_GROUNDSPEED:
		case	PROPERTY_ADV_RADALT:
		case	PROPERTY_TTIP_RADALT:
			*pEnum	=	UNITS_UNKNOWN;
			break;

		case	PROPERTY_TTIP:
		case	PROPERTY_TTIP_ID:
		case	PROPERTY_ADV_ID:
			*pEnum	=	UNITS_STRING;
			break;
		default:
			*pEnum	=	-1;
			return	FALSE;
	}
	return	TRUE;
}

//caution! defines several methods at once
DEFINE_PANEL_CALLBACK_REFCOUNT(MyPanelCCallback)

MyPanelCCallback::MyPanelCCallback(BOOL oldinterface)
{
	m_oldinterface	=	oldinterface;
	//caution! initializes  several vars at once
	INIT_PANEL_CALLBACK_REFCOUNT(MyPanelCCallback);
}

MyPanelCCallback::~MyPanelCCallback(void)
{
}
